<?php
/**
 * Layout Panel customize settings.
 *
 * @package    Community_Builder
 * @subpackage Customizer\Panels
 * @copyright  Copyright (c) 2018, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Brajesh Singh
 */

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Layout Panel helper.
 */
class CB_Customize_Panel_Layout {

	/**
	 * Panel Id.
	 *
	 * @var string
	 */
	private $panel = 'cb_layout';

	/**
	 * CB_Customize_Panel_Layout constructor.
	 */
	public function __construct() {
		add_filter( 'cb_customizer_sections', array( $this, 'add_sections' ) );
	}

	/**
	 * Add sections for this panel.
	 *
	 * @param array $sections sections.
	 *
	 * @return array
	 */
	public function add_sections( $sections ) {

		$new_sections = $this->get_sections();

		return array_merge( $sections, $new_sections );
	}

	/**
	 * Get all sections for this panel.
	 *
	 * @return array
	 */
	public function get_sections() {

		$panel = $this->panel;

		$layout_sections = array();

		/**
		 * Global
		 */
		$layout_sections['layout-global'] = array(
			'panel'   => $panel,
			'title'   => __( 'Global', 'community-builder' ),
			'options' => array(
				'theme-layout'   => array(
					'setting' => array(
						'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_choice' ),
						'default'           => cb_get_default( 'theme-layout' ),
					),
					'control' => array(
						'control_type' => 'CB_Customize_Control_Layout',
						'label'        => __( 'Site Layout', 'community-builder' ),
						'description'  => __( 'General site layout. You can override individual pages from page edit screen.', 'community-builder' ),
					),
				),
				'layout-style'   => array(
					'setting' => array(
						'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_choice' ),
						'default'           => cb_get_default( 'layout-style' ),
					),
					'control' => array(
						'control_type' => 'CB_Customize_Control_Box_Layout',
						'label'        => __( 'Layout Style', 'community-builder' ),
						'description'  => __( 'General site layout style. Default is boxed.', 'community-builder' ),
					),
				),
				'theme-fluid-width' => array(
					'setting' => array(
						'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_float' ),
						'transport'         => 'postMessage',
						'default'           => cb_get_default( 'theme-fluid-width' ),
					),
					'control' => array(
						'control_type'    => 'CB_Customize_Control_Range',
						'input_attrs'     => array(
							'min'  => 30,
							'max'  => 100,
							'step' => 1,
						),
						'label'           => __( 'Fluid Layout Width(in % )', 'community-builder' ),
						'description'     => __( 'Site width for fluid layout.', 'community-builder' ),
						'active_callback' => 'cb_is_layout_fluid',
					),
				),

				'content-width' => array(
					'setting' => array(
						'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_float' ),
						'transport'         => 'postMessage',
						'default'           => cb_get_default( 'content-width' ),
					),
					'control' => array(
						'control_type' => 'CB_Customize_Control_Range',
						'input_attrs'  => array(
							'min'  => 30,
							'max'  => 100,
							'step' => 1,
						),

						'label'       => __( 'Content Width(in % )', 'community-builder' ),
						'description' => __( 'Content area width in percentage of the total page width. Used for 2 column pages.', 'community-builder' ),
					),
				),


				'home-layout' => array(
					'setting' => array(
						'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_choice' ),
						'default'           => cb_get_default( 'home-layout' ),
					),
					'control' => array(
						'control_type' => 'CB_Customize_Control_Page_Layout',
						'label'        => __( 'Home Page Layout', 'community-builder' ),
						'description'  => __( 'Choose layout for home page.', 'community-builder' ),
					),
				),
			),
		);

		/**
		 * Header
		 */
		$layout_sections['header'] = array(
			'panel'   => $panel,
			'title'   => __( 'Header', 'community-builder' ),
			'options' => array(
				'header-layout-social-group-title' => array(
					'control' => array(
						'control_type' => 'CB_Customize_Control_Info_Title',
						'label'        => __( 'Social Settings', 'community-builder' ),
					),
				),
				'header-show-social'               => array(
					'setting' => array(
						'sanitize_callback' => 'absint',
						'default'           => cb_get_default( 'header-show-social' ),
						'transport'         => 'postMessage',
					),
					'control' => array(
						'label' => __( 'Show social icons', 'community-builder' ),
						'type'  => 'checkbox',
						'std'   => 1,
					),
				),
				'header-social-icons'              => array(
					'setting' => array(
						'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_multi_choices' ),
						'transport'         => 'postMessage',
						'default'           => cb_get_default( 'header-social-icons' ),
					),
					'control' => array(
						'control_type'    => 'CB_Customize_Control_Sortable',
						'input_attrs'     => array(),
						'label'           => __( 'Enabled Icons', 'community-builder' ),
						'description'     => __( 'These icons will be visible in header. Please make sure you have saved your social profile in General->Social Profiles.', 'community-builder' ),
						'choices'         => CB_Settings_Choices::get( 'header-social-icons' ),
						'active_callback' => 'cb_is_header_social_icons_enabled',
					),
				),
				'header-social-icon-font-size'     => CB_Customize_Setting_Builder::get_responsive_range_settings(
					array(
						'default'         => cb_get_default( 'header-social-icon-font-size' ),
						'label'           => __( 'Icon Size (px)', 'community-builder' ),
						'active_callback' => 'cb_is_header_social_icons_enabled',
					)
				),
				'custom-text-block-1'              => array(
					'setting' => array(
						'sanitize_callback' => 'wp_kses_data',
						'transport'         => 'postMessage',
						'default'           => cb_get_default( 'custom-text-block-1' ),
					),
					'control' => array(
						'label'           => __( 'Custom Text 1', 'community-builder' ),
						'type'            => 'textarea',
						'active_callback' => 'cb_is_site_header_control_active',
					),
				),
			),
		);

		$layout_sections['header-main-row'] = array(
			'panel'           => $panel,
			'title'           => __( 'Header Main Row', 'community-builder' ),
			'options'         => CB_Customize_Setting_Builder::get_site_header_settings( 'main' ),
			'active_callback' => 'cb_is_site_header_main_row_enabled',
		);

		/**
		 * Footer
		 */
		$layout_sections['footer'] = array(
			'panel'   => $panel,
			'title'   => __( 'Footer', 'community-builder' ),
			'options' => array(
				'footer-enabled-widget-areas'      => array(
					'setting' => array(
						'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_choice' ),
						'default'           => cb_get_default( 'footer-enabled-widget-areas' ),
					),
					'control' => array(
						'control_type' => 'CB_Customize_Control_Radio',
						'label'        => __( 'Number of Widget Areas', 'community-builder' ),
						'mode'         => 'buttonset',
						'choices'      => CB_Settings_Choices::get( 'footer-enabled-widget-areas' ),
					),
				),

				'footer-layout-social-group-title' => array(
					'control' => array(
						'control_type' => 'CB_Customize_Control_Info_Title',
						'label'        => __( 'Social Settings', 'community-builder' ),
					),
				),
				'footer-show-social'               => array(
					'setting' => array(
						'sanitize_callback' => 'absint',
						'default'           => cb_get_default( 'footer-show-social' ),
					),
					'control' => array(
						'label' => __( 'Show social icons', 'community-builder' ),
						'type'  => 'checkbox',
						'std'   => 1,
					),
				),
				'footer-social-icons'              => array(
					'setting' => array(
						'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_multi_choices' ),
						'transport'         => 'refresh',
						'default'           => cb_get_default( 'footer-social-icons' ),
					),
					'control' => array(
						'control_type'    => 'CB_Customize_Control_Sortable',
						'input_attrs'     => array(),
						'label'           => __( 'Enabled Icons', 'community-builder' ),
						'description'     => __( 'These icons will be visible in footer. Please make sure you have saved your social profile in General->Social Profiles.', 'community-builder' ),
						'choices'         => CB_Settings_Choices::get( 'footer-social-icons' ),
						'active_callback' => 'cb_is_footer_social_icons_enabled',
					),
				),
				'footer-social-icon-font-size'     => CB_Customize_Setting_Builder::get_responsive_range_settings(
					array(
						'default'         => cb_get_default( 'footer-social-icon-font-size' ),
						'label'           => __( 'Icon Size (px)', 'community-builder' ),
						'active_callback' => 'cb_is_footer_social_icons_enabled',
					)
				),
			),
		);

		return apply_filters( 'cb_customizer_layout_sections', $layout_sections );
	}

} // end of class.

new CB_Customize_Panel_Layout();
